@@ -24,11 +24,11 @@ Struct::Diff - Recursive diff tools for nested perl structures
 
 =head1 VERSION
 
-Version 0.62
+Version 0.64
 
 =cut
 
-our $VERSION = '0.62';
+our $VERSION = '0.64';
 
 =head1 SYNOPSIS
 
@@ -47,12 +47,14 @@ our $VERSION = '0.62';
     # $dsplit->{a} not exists                       # unchanged omitted, other items originated from $b
     # $dsplit->{b} == {x => [{y => 9}],z => 33};
 
+    dtraverse($d, {callback => sub {print "val $_[0] has status $_[2]" }}); # traverse through diff
+
     patch($a, $diff);
     # $a now equal to $b by structure and data
 
 =head1 EXPORT
 
-Nothing exports by default
+Nothing is exported by default.
 
 =head1 SUBROUTINES
 
@@ -311,20 +313,42 @@ sub dsplit($) {
 
 =head2 dtraverse
 
-Traverse through diff invoking callback functions for subdiff statuses. Important: path (secont argument,
-passed to callback functions) is actual for callback lifetime and will be changed afterwards.
+Traverse through diff invoking callback function for subdiff statuses.
 
     my $opts = {
-        A => sub { print "added:", $_[0], "depth:", @{$_[1]} },
-        U => sub { print "unchaanged: ", $_[0] },
+        callback => sub { print "added value:", $_[0], "depth:", @{$_[1]}, "status:", $_[2] },
+        sortkeys => sub { sort { $a <=> $b } @_ }   # numeric sort for keys under diff
     };
     dtraverse($diff, $opts);
 
+=head3 Available options
+
+=over 4
+
+=item callback
+
+Mandatory option, must contain coderef to callback fuction. Three arguments will be passed to provided
+subroutine: value, path, status. Important: path (second argument) is actual for callback lifetime and will be
+immedeately changed afterwards.
+
+=item sortkeys
+
+Defines how will be traversed subdiffs for hashes. Keys will be picked Randomely (depends on C<keys> behavior,
+default), sorted by provided subroutine (if value is a coderef) or lexically sorted if set to some other true value.
+
+=item statuses
+
+Exact list of statuses. Sequence defines invocation priority.
+
+=back
+
 =cut
 
 sub dtraverse($$;$);
 sub dtraverse($$;$) {
     my ($d, $o, $p) = (shift, shift, shift || []);
+    croak "Callback must be a code reference" unless (ref $o->{'callback'} eq 'CODE');
+    croak "Statuses argument must be ARRAY" if ($o->{'statuses'} and ref $o->{'statuses'} ne 'ARRAY');
     _validate_meta($d);
 
     if (exists $d->{'D'}) {
@@ -335,21 +359,17 @@ sub dtraverse($$;$) {
                 pop @{$p};
             }
         } else { # HASH
-            for my $k (keys %{$d->{'D'}}) {
+            my @keys = keys %{$d->{'D'}};
+            @keys = ref $o->{'sortkeys'} eq 'CODE' ? $o->{'sortkeys'}(@keys) : sort @keys if ($o->{'sortkeys'});
+            for my $k (@keys) {
                 push @{$p}, { 'keys' => [$k] };
                 dtraverse($d->{'D'}->{$k}, $o, $p);
                 pop @{$p};
             }
         }
-    } elsif (exists $d->{'U'} and $o->{'U'}) {
-        $o->{'U'}($d->{'U'}, $p);
-    } elsif (exists $d->{'A'} and $o->{'A'}) {
-        $o->{'A'}($d->{'A'}, $p);
-    } elsif (exists $d->{'R'} and $o->{'R'}) {
-        $o->{'R'}($d->{'R'}, $p);
     } else {
-        $o->{'N'}($d->{'N'}, $p) if (exists $d->{'N'} and $o->{'N'});
-        $o->{'O'}($d->{'O'}, $p) if (exists $d->{'O'} and $o->{'O'});
+        my @statuses = $o->{'statuses'} ? @{$o->{'statuses'}} : keys %{$d};
+        map { $o->{'callback'}($d->{$_}, $p, $_) if exists $d->{$_} } @statuses;
     }
 }
 
